﻿using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Windows.Forms;
using Infragistics.Win.UltraWinGrid;
using PropertyChangedEventArgs = System.ComponentModel.PropertyChangedEventArgs;
using PropertyChangedEventHandler = System.ComponentModel.PropertyChangedEventHandler;

namespace InfragisticsBindings
{
    public partial class Form1 : Form, INotifyPropertyChanged
    {
        public BindingList<DataObject> DataObjects;

        private DataObject _selecectDataObject;

        public DataObject SelectedDataObject
        {
            get { return _selecectDataObject; }
            set
            {
                _selecectDataObject = value;
                NotifyPropertyChanged();
            }
        }

        private DataObject _selectedComboDataObject;
        public DataObject SelectedComboDataObject
        {
            get { return _selectedComboDataObject; }
            set
            {
                _selectedComboDataObject = value;
                NotifyPropertyChanged();
            }
        }

        public Form1()
        {
            InitializeComponent();

            DataObjects = new BindingList<DataObject>();
            DataObjects.Add(new DataObject() { Prop1 = "A", Prop2 = "B" });
            DataObjects.Add(new DataObject() { Prop1 = "C", Prop2 = "D" });

            ultraComboEditor1.SetDataBinding(DataObjects, null);
            ultraComboEditor1.ValueMember = Infragistics.Win.BindableValueList.USE_LISTOBJECT_AS_VALUEMEMBER;
            ultraComboEditor1.DisplayMember = "Prop2";
            Binding b = new Binding("Value", this, "SelectedDataObject", false, DataSourceUpdateMode.OnPropertyChanged);
            ultraComboEditor1.DataBindings.Add(b);

            SelectedDataObject = DataObjects.First();

            ultraCombo1.DisplayMember = "Prop2";
            ultraCombo1.SetDataBinding(DataObjects, null);

            // *** Infragistics
            // USE_LISTOBJECT_AS_VALUEMEMBER is only supported for UlktraComboEditor. It is not supported for 
            // UltraCombo. But you can handle this using an unbound colmun that contains the ListObject. 
            //
            //ultraCombo1.ValueMember = Infragistics.Win.BindableValueList.USE_LISTOBJECT_AS_VALUEMEMBER;
            ultraCombo1.ValueMember = "MyListObjectColumn";

            Binding c = new Binding("Value", this, "SelectedComboDataObject", false, DataSourceUpdateMode.OnPropertyChanged);
            ultraCombo1.DataBindings.Add(c);

            ultraLabel1.DataBindings.Add("Text", this, "SelectedDataObject.Prop1");
            ultraLabel4.DataBindings.Add("Text", this, "SelectedComboDataObject.Prop1");
        }

        private void ultraButton1_Click(object sender, System.EventArgs e)
        {
            DataObjects.Add(new DataObject() { Prop1 = "E", Prop2 = "F" });
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void NotifyPropertyChanged([CallerMemberName] string propertyName = null)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
        }

        private void ultraComboEditor1_ValueChanged(object sender, System.EventArgs e)
        {
            //SelectedDataObject = ultraComboEditor1.SelectedItem.ListObject as DataObject;
        }

        // *** Infragistics
        private void ultraCombo1_InitializeLayout(object sender, Infragistics.Win.UltraWinGrid.InitializeLayoutEventArgs e)
        {
            UltraGridLayout layout = e.Layout;
            UltraGridBand band = layout.Bands[0];

            // Add an unbound column to each row that contains the ListObject (DataObject)
            if (false == band.Columns.Exists("MyListObjectColumn"))
            {
                UltraGridColumn myListObjectColumn = band.Columns.Add("MyListObjectColumn");
                myListObjectColumn.DataType = typeof(DataObject);
            }
        }

        // *** Infragistics
        private void ultraCombo1_InitializeRow(object sender, InitializeRowEventArgs e)
        {
            // Populate the unbound column with the ListObject (DataObject) of the row. 
            if (e.Row.Cells.Exists("MyListObjectColumn"))
            {
                e.Row.Cells["MyListObjectColumn"].Value = e.Row.ListObject;
            }
        }
    }

    public class DataObject
    {
        public string Prop1 { get; set; }
        public string Prop2 { get; set; }
    }
}
